#include "externs.h"

void F14_initialize ()
/**************/
{
  register T35_twohalves help;
  register T37_memoryword mhelp;
#ifdef M_XENIX
  T37_memoryword near *pt;
  T37_memoryword huge *mptr;
#else
  T37_memoryword *pt;
  register T37_memoryword *mptr;
#endif

#ifdef TRIPTEX
  m29975= &mem[2975 - C2_memmin];
  m29976= &mem[2976 - C2_memmin];
  m29977= &mem[2977 - C2_memmin];
  m29978= &mem[2978 - C2_memmin];
  m29979= &mem[2979 - C2_memmin];
  m29980= &mem[2980 - C2_memmin];
  m29981= &mem[2981 - C2_memmin];
  m29982= &mem[2982 - C2_memmin];
  m29983= &mem[2983 - C2_memmin];
  m29984= &mem[2984 - C2_memmin];
  m29985= &mem[2985 - C2_memmin];
  m29986= &mem[2986 - C2_memmin];
  m29987= &mem[2987 - C2_memmin];
  m29988= &mem[2988 - C2_memmin];
  m29989= &mem[2989 - C2_memmin];
  m29990= &mem[2990 - C2_memmin];
  m29991= &mem[2991 - C2_memmin];
  m29992= &mem[2992 - C2_memmin];
  m29993= &mem[2993 - C2_memmin];
  m29994= &mem[2994 - C2_memmin];
  m29995= &mem[2995 - C2_memmin];
  m29996= &mem[2996 - C2_memmin];
  m29997= &mem[2997 - C2_memmin];
  m29998= &mem[2998 - C2_memmin];
  m29999= &mem[2999 - C2_memmin];
  m30000= &mem[3000 - C2_memmin];
#else
  m29975= &mem[29975 - C2_memmin];
  m29976= &mem[29976 - C2_memmin];
  m29977= &mem[29977 - C2_memmin];
  m29978= &mem[29978 - C2_memmin];
  m29979= &mem[29979 - C2_memmin];
  m29980= &mem[29980 - C2_memmin];
  m29981= &mem[29981 - C2_memmin];
  m29982= &mem[29982 - C2_memmin];
  m29983= &mem[29983 - C2_memmin];
  m29984= &mem[29984 - C2_memmin];
  m29985= &mem[29985 - C2_memmin];
  m29986= &mem[29986 - C2_memmin];
  m29987= &mem[29987 - C2_memmin];
  m29988= &mem[29988 - C2_memmin];
  m29989= &mem[29989 - C2_memmin];
  m29990= &mem[29990 - C2_memmin];
  m29991= &mem[29991 - C2_memmin];
  m29992= &mem[29992 - C2_memmin];
  m29993= &mem[29993 - C2_memmin];
  m29994= &mem[29994 - C2_memmin];
  m29995= &mem[29995 - C2_memmin];
  m29996= &mem[29996 - C2_memmin];
  m29997= &mem[29997 - C2_memmin];
  m29998= &mem[29998 - C2_memmin];
  m29999= &mem[29999 - C2_memmin];
  m30000= &mem[30000 - C2_memmin];
  m_30000= &mem[-30000 - C2_memmin];
#endif
  interaction = 3;
  deletionsallowed = TRUE;
  errorcount = 0;
  helpptr = 0;
  useerrhelp = FALSE;
  interrupt = 0;
  oktointerrupt = TRUE;
#if defined(INITEX) || defined(TRIPTEX)
  wasmemend = C2_memmin;
  waslomax = C2_memmin;
  washimin = C1_memmax;
  panicking = FALSE;
# endif

  nestptr = 0;
  maxneststack = 0;
  curlist.modefield = 1;
# ifdef TRIPTEX
  curlist.headfield = 2999;
  curlist.tailfield = 2999;
# else
  curlist.headfield = 29999;
  curlist.tailfield = 29999;
# endif
  curlist.auxfield = -65536000;
  curlist.mlfield = 0;
  curlist.pgfield = 0;
  shownmode = 0;
  pagecontents = 0;
# ifdef TRIPTEX
  pagetail = 2998;
  mem[2998 - C2_memmin]._TAG._TAG3.hh.rh = (int)(-32768);
# else
  pagetail = 29998;
  m29998->_TAG._TAG3.hh.rh = (int)(-32768);
# endif
  lastglue = 32767;
  lastpenalty = 0;
  lastkern = 0;
  pagesofar[7] = 0;
  pagemaxdepth = 0;
  {
    register int k;
    for (k = 5267; k <= 5976; k++)
      xeqlevel[k - 5267] = -127;
  }
  nonewcontrolsequence = TRUE;
  hash[0]._TAG._TAG1.lh = 0;
  hash[0].rh = 0;
  help = hash[0];
  {
    register int k;
    for (k = 1; k <= 3266; k++)
      hash[k]=help;
  }
  saveptr = 0;
  curlevel = -127;
  curgroup = 0;
  curboundary = 0;
  maxsavestack = 0;
  magset = 0;
  curmark[0] = (int)(-32768);
  curmark[1] = (int)(-32768);
  curmark[2] = (int)(-32768);
  curmark[3] = (int)(-32768);
  curmark[4] = (int)(-32768);
  curval = 0;
  curvallevel = 0;
  radix = 0;
  {
    register int k;
    for (k = 0; k <= 16; k++)
      readopen[k] = 2;
  }
  condptr = (int)(-32768);
  iflimit = 0;
  curif = 0;
  ifline = 0;
  strcpy (texformatdefault, "plain.fmt");
  {
    register int k;
    for (k = 0; k <= C9_fontmax; k++)
      fontused[k] = FALSE;
  }
  nullcharacter.b0 = -128;
  nullcharacter.b1 = -128;
  nullcharacter.b2 = -128;
  nullcharacter.b3 = -128;
  totalpages = 0;
  maxv = 0;
  maxh = 0;
  maxpush = 0;
  lastbop = -1;
  doingleaders = FALSE;
  deadcycles = 0;
  dvilimit = C18_dvibufsize;
  dviptr = 0;
  dvioffset = 0;
  dvigone = 0;
  downptr = (int)(-32768);
  rightptr = (int)(-32768);
  adjusttail = (int)(-32768);
  packbeginline = 0;
  emptyfield.rh = 0;
  emptyfield._TAG._TAG1.lh = (int)(-32768);
  nulldelimiter.b0 = 0;
  nulldelimiter.b1 = -128;
  nulldelimiter.b2 = 0;
  nulldelimiter.b3 = -128;
  alignptr = (int)(-32768);
  curalign = (int)(-32768);
  curspan = (int)(-32768);
  curloop = (int)(-32768);
  curhead = (int)(-32768);
  curtail = (int)(-32768);
  {
    register int z;
    for (z = 0; z <= 307; z++)
    {
      hyphword[z] = 0;
      hyphlist[z] = (int)(-32768);
    }
  }
  hyphcount = 0;
  outputactive = FALSE;
  insertpenalties = 0;
  aftertoken = 0;
  longhelpseen = FALSE;
  formatident = 0;
  {
    register int k;
    for (k = 0; k <= 17; k++)
      writeopen[k] = FALSE;
  }
  editnamestart = 0;
# ifdef INITEX
  {
    register int k;
    for (k = -29999; k <= -29981; k++)
      mem[k - C2_memmin]._TAG._TAG1._int = 0;
    k = -30000;
    mhelp._TAG._TAG3.hh.rh = -32767;
    mhelp._TAG._TAG3.hh._TAG._TAG2.b0 = 0;
    mhelp._TAG._TAG3.hh._TAG._TAG2.b1 = 0;
    while (k <= -29981)
    {
      mem[k - C2_memmin] = mhelp;
      k += 4;
    }
  }
  mem[-29994 - C2_memmin]._TAG._TAG1._int = 65536;
  mem[-29996 - C2_memmin]._TAG._TAG3.hh._TAG._TAG2.b0 = 1;
  mem[-29990 - C2_memmin]._TAG._TAG1._int = 65536;
  mem[-29992 - C2_memmin]._TAG._TAG3.hh._TAG._TAG2.b0 = 2;
  mem[-29986 - C2_memmin]._TAG._TAG1._int = 65536;
  mem[-29988 - C2_memmin]._TAG._TAG3.hh._TAG._TAG2.b0 = 1;
  mem[-29985 - C2_memmin]._TAG._TAG1._int = 65536;
  mem[-29988 - C2_memmin]._TAG._TAG3.hh._TAG._TAG2.b1 = 1;
  mem[-29982 - C2_memmin]._TAG._TAG1._int = -65536;
  mem[-29984 - C2_memmin]._TAG._TAG3.hh._TAG._TAG2.b0 = 1;
  rover = -29980;
  mem[rover - C2_memmin]._TAG._TAG3.hh.rh = 32767;
  mem[rover - C2_memmin]._TAG._TAG3.hh._TAG._TAG1.lh = 1000;
  mem[rover + 1 - C2_memmin]._TAG._TAG3.hh._TAG._TAG1.lh = rover;
  mem[rover + 1 - C2_memmin]._TAG._TAG3.hh.rh = rover;
  lomemmax = rover + 1000;
  mem[lomemmax - C2_memmin]._TAG._TAG3.hh.rh = (int)(-32768);
  mem[lomemmax - C2_memmin]._TAG._TAG3.hh._TAG._TAG1.lh = (int)(-32768);
  mhelp=mem[lomemmax - C2_memmin];
  mptr= &mem[29987 - C2_memmin];
  {
    register int k;
    for (k = 29987; k <= 30000; k++)
      *mptr++ =mhelp;
  }
  m29990->_TAG._TAG3.hh._TAG._TAG1.lh = 7359;
  m29991->_TAG._TAG3.hh.rh = 128;
  m29991->_TAG._TAG3.hh._TAG._TAG1.lh = (int)(-32768);
  m29993->_TAG._TAG3.hh._TAG._TAG2.b0 = 1;
  m29993->_TAG._TAG3.hh._TAG._TAG2.b1 = 0;
  m29994->_TAG._TAG3.hh._TAG._TAG1.lh = 32767;
  m30000->_TAG._TAG3.hh._TAG._TAG2.b1 = 127;
  m30000->_TAG._TAG3.hh._TAG._TAG2.b0 = 1;
  m30000->_TAG._TAG3.hh.rh = 30000;
  m29998->_TAG._TAG3.hh._TAG._TAG2.b0 = 10;
  m29998->_TAG._TAG3.hh._TAG._TAG2.b1 = 0;
  avail = (int)(-32768);
  memend = 30000;
  himemmin = 29987;
  varused = 20;
  dynused = 14;
  eqtb[3525]._TAG._TAG3.hh._TAG._TAG2.b0 = 100;
  eqtb[3525]._TAG._TAG3.hh.rh = (int)(-32768);
  eqtb[3525]._TAG._TAG3.hh._TAG._TAG2.b1 = -128;
  mhelp = eqtb[3525];
  pt = &eqtb[1];
  {
    register int k;
    for (k = 1; k <= 3524; k++)
      *pt++ = mhelp;
  }
  eqtb[3526]._TAG._TAG3.hh.rh = -30000;
  eqtb[3526]._TAG._TAG3.hh._TAG._TAG2.b1 = -127;
  eqtb[3526]._TAG._TAG3.hh._TAG._TAG2.b0 = 116;
  mhelp = eqtb[3526];
  pt = &eqtb[3527];
  {
    register int k;
    for (k = 3527; k <= 4055; k++)
      *pt++ = mhelp;
  }
  m_30000->_TAG._TAG3.hh.rh += 530;
  eqtb[4056]._TAG._TAG3.hh.rh = (int)(-32768);
  eqtb[4056]._TAG._TAG3.hh._TAG._TAG2.b0 = 117;
  eqtb[4056]._TAG._TAG3.hh._TAG._TAG2.b1 = -127;
  mhelp = eqtb[3525];
  pt = &eqtb[4057];
  {
    register int k;
    for (k = 4057; k <= 4321; k++)
      *pt++ =mhelp;
  }
  eqtb[4322]._TAG._TAG3.hh.rh = (int)(-32768);
  eqtb[4322]._TAG._TAG3.hh._TAG._TAG2.b0 = 118;
  eqtb[4322]._TAG._TAG3.hh._TAG._TAG2.b1 = -127;
  mhelp = eqtb[4322];
  pt = &eqtb[4323];
  {
    register int k;
    for (k = 4323; k <= 4577; k++)
      *pt++ =mhelp;
  }
  eqtb[4578]._TAG._TAG3.hh.rh = 0;
  eqtb[4578]._TAG._TAG3.hh._TAG._TAG2.b0 = 119;
  eqtb[4578]._TAG._TAG3.hh._TAG._TAG2.b1 = -127;
  mhelp = eqtb[4578];
  pt = &eqtb[4579];
  {
    register int k;
    for (k = 4579; k <= 4626; k++)
      *pt++ =mhelp;
  }
  eqtb[4627]._TAG._TAG3.hh.rh = 0;
  eqtb[4627]._TAG._TAG3.hh._TAG._TAG2.b0 = 119;
  eqtb[4627]._TAG._TAG3.hh._TAG._TAG2.b1 = -127;
  mhelp = eqtb[4627];
  pt = &eqtb[4628];
  {
    register int k;
    for (k = 4628; k <= 5266; k++)
      *pt++ =mhelp;
    for (k = 0; k <= 127; k++)
    {
      eqtb[4627 + k]._TAG._TAG3.hh.rh = 12;
      eqtb[5139 + k]._TAG._TAG3.hh.rh = k - 32768;
      eqtb[5011 + k]._TAG._TAG3.hh.rh = 1000;
    }
  }
  eqtb[4640]._TAG._TAG3.hh.rh = 5;
  eqtb[4659]._TAG._TAG3.hh.rh = 10;
  eqtb[4719]._TAG._TAG3.hh.rh = 0;
  eqtb[4664]._TAG._TAG3.hh.rh = 14;
  eqtb[4754]._TAG._TAG3.hh.rh = 15;
  eqtb[4627]._TAG._TAG3.hh.rh = 9;
  {
    register int k;
    for (k = 48; k <= 57; k++)
      eqtb[5139 + k]._TAG._TAG3.hh.rh = k - 4096;
    for (k = 65; k <= 90; k++)
    {
      eqtb[4627 + k]._TAG._TAG3.hh.rh = 11;
      eqtb[4659 + k]._TAG._TAG3.hh.rh = 11;
      eqtb[5139 + k]._TAG._TAG3.hh.rh = k - 3840;
      eqtb[5171 + k]._TAG._TAG3.hh.rh = k - 3808;
      eqtb[4755 + k]._TAG._TAG3.hh.rh = k + 32;
      eqtb[4787 + k]._TAG._TAG3.hh.rh = k + 32;
      eqtb[4883 + k]._TAG._TAG3.hh.rh = k;
      eqtb[4915 + k]._TAG._TAG3.hh.rh = k;
      eqtb[5011 + k]._TAG._TAG3.hh.rh = 999;
    }
    pt = &eqtb[5267];
    for (k = 5267; k <= 5572; k++)
      (pt++)->_TAG._TAG1._int = 0;
  }
  eqtb[5284]._TAG._TAG1._int = 1000;
  eqtb[5268]._TAG._TAG1._int = 10000;
  eqtb[5308]._TAG._TAG1._int = 1;
  eqtb[5307]._TAG._TAG1._int = 25;
  eqtb[5312]._TAG._TAG1._int = 92;
  eqtb[5315]._TAG._TAG1._int = 13;
  {
    register int k;
    pt = &eqtb[5573];
    for (k = 5573; k <= 5700; k++)
      (pt++)->_TAG._TAG1._int = -1;
    pt = &eqtb[5701];
    for (k = 5701; k <= 5976; k++)
      (pt++)->_TAG._TAG1._int = 0;
  }
  eqtb[5619]._TAG._TAG1._int = 0;
  hashused = 3258;
  cscount = 0;
  eqtb[3267]._TAG._TAG3.hh._TAG._TAG2.b0 = 115;
  hash[3267 - 258].rh = 362;
  fontptr = 0;
  fmemptr = 7;
  fontname[0] = 654;
  fontarea[0] = 207;
  hyphenchar[0] = 45;
  skewchar[0] = -1;
  fontbc[0] = 1;
  fontec[0] = 0;
  fontsize[0] = 0;
  fontdsize[0] = 0;
  charbase[0] = 0;
  widthbase[0] = 0;
  heightbase[0] = 0;
  depthbase[0] = 0;
  italicbase[0] = 0;
  ligkernbase[0] = 0;
  kernbase[0] = 0;
  extenbase[0] = 0;
  fontglue[0] = (int)(-32768);
  fontparams[0] = 7;
  parambase[0] = -1;
  {
    register int k;
    for (k = 0; k <= 6; k++)
      fontinfo[k]._TAG._TAG1._int = 0;
  }
  trieopptr = -128;
  trie[0].rh = 0;
  trie[0]._TAG._TAG2.b1 = 0;
  trie[0]._TAG._TAG2.b0 = 0;
  help = trie[0];
  {
    register int k;
    for (k = 1; k <= 127; k++)
      trie[k]=help;
  }
  triemax = 127;
  hash[3258 - 258].rh = 1038;
  formatident = 1103;
  hash[3266 - 258].rh = 1138;
  eqtb[3266]._TAG._TAG3.hh._TAG._TAG2.b1 = -127;
  eqtb[3266]._TAG._TAG3.hh._TAG._TAG2.b0 = 112;
  eqtb[3266]._TAG._TAG3.hh.rh = (int)(-32768);
# endif
# ifdef TRIPTEX
  {
    register int k;
    for (k = 2; k <= 20; k++)
      mem[k - C2_memmin]._TAG._TAG1._int = 0;
    k = 1;
    while (k <= 20)
    {
      mem[k - C2_memmin]._TAG._TAG3.hh.rh = -32767;
      mem[k - C2_memmin]._TAG._TAG3.hh._TAG._TAG2.b0 = 0;
      mem[k - C2_memmin]._TAG._TAG3.hh._TAG._TAG2.b1 = 0;
      k = k + 4;
    }
  }
  mem[7- C2_memmin]._TAG._TAG1._int = 65536;
  mem[5 - C2_memmin]._TAG._TAG3.hh._TAG._TAG2.b0 = 1;
  mem[11 - C2_memmin]._TAG._TAG1._int = 65536;
  mem[9 - C2_memmin]._TAG._TAG3.hh._TAG._TAG2.b0 = 2;
  mem[15 - C2_memmin]._TAG._TAG1._int = 65536;
  mem[13 - C2_memmin]._TAG._TAG3.hh._TAG._TAG2.b0 = 1;
  mem[16 - C2_memmin]._TAG._TAG1._int = 65536;
  mem[13 - C2_memmin]._TAG._TAG3.hh._TAG._TAG2.b1 = 1;
  mem[19 - C2_memmin]._TAG._TAG1._int = -65536;
  mem[17 - C2_memmin]._TAG._TAG3.hh._TAG._TAG2.b0 = 1;
  rover = 21;
  mem[rover - C2_memmin]._TAG._TAG3.hh.rh = 32767;
  mem[rover - C2_memmin]._TAG._TAG3.hh._TAG._TAG1.lh = 1000;
  mem[rover + 1 - C2_memmin]._TAG._TAG3.hh._TAG._TAG1.lh = rover;
  mem[rover + 1 - C2_memmin]._TAG._TAG3.hh.rh = rover;
  lomemmax = rover + 1000;
  mem[lomemmax - C2_memmin]._TAG._TAG3.hh.rh = (int)(-32768);
  mem[lomemmax - C2_memmin]._TAG._TAG3.hh._TAG._TAG1.lh = (int)(-32768);
  {
    register int k;
    for (k = 2987; k <= 3000; k++)
      mem[k - C2_memmin]=mem[lomemmax - C2_memmin];
  }
  mem[2990 - C2_memmin]._TAG._TAG3.hh._TAG._TAG1.lh = 7359;
  mem[2991 - C2_memmin]._TAG._TAG3.hh.rh = 128;
  mem[2991 - C2_memmin]._TAG._TAG3.hh._TAG._TAG1.lh = (int)(-32768);
  mem[2993 - C2_memmin]._TAG._TAG3.hh._TAG._TAG2.b0 = 1;
  mem[2994 - C2_memmin]._TAG._TAG3.hh._TAG._TAG1.lh = 32767;
  mem[2993 - C2_memmin]._TAG._TAG3.hh._TAG._TAG2.b1 = 0;
  mem[3000 - C2_memmin]._TAG._TAG3.hh._TAG._TAG2.b1 = 127;
  mem[3000 - C2_memmin]._TAG._TAG3.hh._TAG._TAG2.b0 = 1;
  mem[3000 - C2_memmin]._TAG._TAG3.hh.rh = 3000;
  mem[2998 - C2_memmin]._TAG._TAG3.hh._TAG._TAG2.b0 = 10;
  mem[2998 - C2_memmin]._TAG._TAG3.hh._TAG._TAG2.b1 = 0;
  avail = (int)(-32768);
  memend = 3000;
  himemmin = 2987;
  varused = 20;
  dynused = 14;
  eqtb[3525]._TAG._TAG3.hh._TAG._TAG2.b0 = 100;
  eqtb[3525]._TAG._TAG3.hh.rh = (int)(-32768);
  eqtb[3525]._TAG._TAG3.hh._TAG._TAG2.b1 = -128;
  {
    register int k;
    for (k = 1; k <= 3524; k++)
      eqtb[k]=eqtb[3525];
  }
  eqtb[3526]._TAG._TAG3.hh.rh = 1;
  eqtb[3526]._TAG._TAG3.hh._TAG._TAG2.b1 = -127;
  eqtb[3526]._TAG._TAG3.hh._TAG._TAG2.b0 = 116;
  {
    register int k;
    for (k = 3527; k <= 4055; k++)
      eqtb[k]=eqtb[3526];
  }
  mem[1 - C2_memmin]._TAG._TAG3.hh.rh = mem[1 - C2_memmin]._TAG._TAG3.hh.rh + 530;
  eqtb[4056]._TAG._TAG3.hh.rh = (int)(-32768);
  eqtb[4056]._TAG._TAG3.hh._TAG._TAG2.b0 = 117;
  eqtb[4056]._TAG._TAG3.hh._TAG._TAG2.b1 = -127;
  {
    register int k;
    for (k = 4057; k <= 4321; k++)
      eqtb[k]=eqtb[3525];
  }
  eqtb[4322]._TAG._TAG3.hh.rh = (int)(-32768);
  eqtb[4322]._TAG._TAG3.hh._TAG._TAG2.b0 = 118;
  eqtb[4322]._TAG._TAG3.hh._TAG._TAG2.b1 = -127;
  {
    register int k;
    for (k = 4323; k <= 4577; k++)
      eqtb[k]=eqtb[4322];
  }
  eqtb[4578]._TAG._TAG3.hh.rh = 0;
  eqtb[4578]._TAG._TAG3.hh._TAG._TAG2.b0 = 119;
  eqtb[4578]._TAG._TAG3.hh._TAG._TAG2.b1 = -127;
  {
    register int k;
    for (k = 4579; k <= 4626; k++)
      eqtb[k]=eqtb[4578];
  }
  eqtb[4627]._TAG._TAG3.hh.rh = 0;
  eqtb[4627]._TAG._TAG3.hh._TAG._TAG2.b0 = 119;
  eqtb[4627]._TAG._TAG3.hh._TAG._TAG2.b1 = -127;
  {
    register int k;
    for (k = 4628; k <= 5266; k++)
      eqtb[k]=eqtb[4627];
    for (k = 0; k <= 127; k++)
    {
      eqtb[4627 + k]._TAG._TAG3.hh.rh = 12;
      eqtb[5139 + k]._TAG._TAG3.hh.rh = k - 32768;
      eqtb[5011 + k]._TAG._TAG3.hh.rh = 1000;
    }
  }
  eqtb[4640]._TAG._TAG3.hh.rh = 5;
  eqtb[4659]._TAG._TAG3.hh.rh = 10;
  eqtb[4719]._TAG._TAG3.hh.rh = 0;
  eqtb[4664]._TAG._TAG3.hh.rh = 14;
  eqtb[4754]._TAG._TAG3.hh.rh = 15;
  eqtb[4627]._TAG._TAG3.hh.rh = 9;
  {
    register int k;
    for (k = 48; k <= 57; k++)
      eqtb[5139 + k]._TAG._TAG3.hh.rh = k - 4096;
    for (k = 65; k <= 90; k++)
    {
      eqtb[4627 + k]._TAG._TAG3.hh.rh = 11;
      eqtb[(4627 + k) + 32]._TAG._TAG3.hh.rh = 11;
      eqtb[5139 + k]._TAG._TAG3.hh.rh = k - 3840;
      eqtb[(5139 + k) + 32]._TAG._TAG3.hh.rh = k - 3808;
      eqtb[4755 + k]._TAG._TAG3.hh.rh = k + 32;
      eqtb[(4755 + k) + 32]._TAG._TAG3.hh.rh = k + 32;
      eqtb[4883 + k]._TAG._TAG3.hh.rh = k;
      eqtb[(4883 + k) + 32]._TAG._TAG3.hh.rh = k;
      eqtb[5011 + k]._TAG._TAG3.hh.rh = 999;
    }
    for (k = 5267; k <= 5572; k++)
      eqtb[k]._TAG._TAG1._int = 0;
  }
  eqtb[5284]._TAG._TAG1._int = 1000;
  eqtb[5268]._TAG._TAG1._int = 10000;
  eqtb[5308]._TAG._TAG1._int = 1;
  eqtb[5307]._TAG._TAG1._int = 25;
  eqtb[5312]._TAG._TAG1._int = 92;
  eqtb[5315]._TAG._TAG1._int = 13;
  {
    register int k;
    for (k = 0; k <= 127; k++)
      eqtb[5573 + k]._TAG._TAG1._int = -1;
    eqtb[5619]._TAG._TAG1._int = 0;
    for (k = 5701; k <= 5976; k++)
      eqtb[k]._TAG._TAG1._int = 0;
  }
  hashused = 3258;
  cscount = 0;
  eqtb[3267]._TAG._TAG3.hh._TAG._TAG2.b0 = 115;
  hash[3267 - 258].rh = 362;
  fontptr = 0;
  fmemptr = 7;
  fontname[0] = 654;
  fontarea[0] = 207;
  hyphenchar[0] = 45;
  skewchar[0] = -1;
  fontbc[0] = 1;
  fontec[0] = 0;
  fontsize[0] = 0;
  fontdsize[0] = 0;
  charbase[0] = 0;
  widthbase[0] = 0;
  heightbase[0] = 0;
  depthbase[0] = 0;
  italicbase[0] = 0;
  ligkernbase[0] = 0;
  kernbase[0] = 0;
  extenbase[0] = 0;
  fontglue[0] = (int)(-32768);
  fontparams[0] = 7;
  parambase[0] = -1;
  {
    register int k;
    for (k = 0; k <= 6; k++)
      fontinfo[k]._TAG._TAG1._int = 0;
  }
  trieopptr = -128;
  trie[0].rh = 0;
  trie[0]._TAG._TAG2.b1 = 0;
  trie[0]._TAG._TAG2.b0 = 0;
  {
    register int k;
    for (k = 1; k <= 127; k++)
      trie[k]=trie[0];
  }
  triemax = 127;
  hash[3258 - 258].rh = 1038;
  formatident = 1103;
  hash[3266 - 258].rh = 1138;
  eqtb[3266]._TAG._TAG3.hh._TAG._TAG2.b1 = -127;
  eqtb[3266]._TAG._TAG3.hh._TAG._TAG2.b0 = 112;
  eqtb[3266]._TAG._TAG3.hh.rh = (int)(-32768);
# endif
}


void F15_println ()
/***********/
{
  switch (selector)
  {
    case 19: 
      printf ("\n");
      fprintf (logfile.fp, "\n");
      termoffset = 0;
      fileoffset = 0;
      break;
    case 18: 
      fprintf (logfile.fp, "\n");
      fileoffset = 0;
      break;
    case 17: 
      printf ("\n");
      termoffset = 0;
      break;
    case 16: 
    case 20: 
    case 21: 
      break;
    default: 
      fprintf (writefile[selector].fp, "\n");
      break;
  }
}

void F16_printchar (s)
/**************/
T21_asciicode s;
{
  if (s == (int)eqtb[5316]._TAG._TAG1._int)
  {
    if (selector < 20)
    {
      F15_println ();
      return;
    }
  }
  switch (selector)
  {
    case 19: 
      fwrite(&s,1,1,stdout);
      fwrite(&s,1,1,logfile.fp);
      if (++termoffset == C6_maxprintline)
      {
	printf ("\n");
	termoffset = 0;
      }
      if (++fileoffset == C6_maxprintline)
      {
	fprintf (logfile.fp, "\n");
	fileoffset = 0;
      }
      break;
    case 18: 
      fwrite(&s,1,1,logfile.fp);
      if (++fileoffset == C6_maxprintline)
	F15_println ();
      break;
    case 17: 
      fwrite(&s,1,1,stdout);
      if (++termoffset == C6_maxprintline)
	F15_println ();
      break;
    case 16: 
      break;
    case 20: 
      if (tally < trickcount)
	trickbuf[tally % C4_errorline] = s;
      break;
    case 21: 
      if (poolptr < C15_poolsize)
	strpool[poolptr++] = s;
      break;
    default: 
      fwrite(&s,1,1,writefile[selector].fp);
      break;
  }
  tally++;
}

void F17_print (s)
/**********/
register int s;
{
#ifdef VAX
  register int j, help;
#else
  register T25_poolpointer j, help;
#endif

  if (s >= strptr)
    s = 131;
  else
    if (s < 128)
      if (s < 0)
	s = 131;
      else
	if (s == (int)eqtb[5316]._TAG._TAG1._int)
	{
	  if (selector < 20)
	  {
	    F15_println ();
	    return;
	  }
	}
  j = strstart[s];
  help = strstart[s + 1];
  while (j < help)
    F16_printchar (strpool[j++]);
}

void F18_slowprint (s)
/**************/
register int   s;
{
#ifdef VAX
  register int j,help;
#else
  register T25_poolpointer j, help;
#endif

  if (s >= strptr)
    s = 131;
  else
    if (s < 128)
      if (s < 0)
	s = 131;
      else
	if (s == (int)eqtb[5316]._TAG._TAG1._int)
	{
	  if (selector < 20)
	  {
	    F15_println ();
	    return;
	  }
	}
  j = strstart[s];
  help = strstart[s + 1];
  while (j < help)
    F17_print ((int)strpool[j++]);
}

void F19_printnl (s)
/************/
#ifdef VAX
register int s;
#else
register T26_strnumber s;
#endif
{
  if (((termoffset > 0) && (selector&1)) || ((fileoffset > 0) && (selector >= 18)))
    F15_println ();
  F17_print (s);
}

void F20_printesc (s)
/*************/
#ifdef VAX
register int s;
#else
register T26_strnumber s;
#endif
{
  register int   c;

  c = (int)eqtb[5312]._TAG._TAG1._int;
  if ((c >= 0) && (c < 128))
    F17_print (c);
  F17_print (s);
}

void F21_printthedigs (k)
/*****************/
#ifdef VAX
register int k;
#else
register T22_eightbits k;
#endif
{
#ifdef VAX
  register int help;
#else
  register char help;
#endif
  while (k > 0)
    if ((help=dig[--k]) < 10)
      F16_printchar (48 + help);
    else
      F16_printchar (55 + help);
}

void F22_printint (n)
/*************/
#ifdef VAX
register int n;
#else
long n;
#endif
{
  register int k;
#ifdef VAX
register int m;
#else
  long m;
#endif

  k = 0;
  if (n < 0L)
  {
    F16_printchar ((char)45);
    if (n > (-100000000))
      n = -n;
    else
    {
      m = (-1) - n;
      n = m / 10;
      m = (m % 10) + 1;
      k = 1;
      if (m < 10L)
	dig[0] = m;
      else
      {
	dig[0] = 0;
	n++;
      }
    }
  }
  do
  {
    dig[k++] = n % 10;
    n /= 10;
  }
  while (n != 0);
  F21_printthedigs ((T22_eightbits)k);
}

void F23_printcs (p)
/************/
register int   p;
{
  if (p < 258)
    if (p >= 129)
      if (p == 257)
      {
	F20_printesc (364);
	F20_printesc (365);
      }
      else
      {
	F20_printesc (p - 129);
	if (eqtb[4498 + p]._TAG._TAG3.hh.rh == 11)
	  F16_printchar ((char)32);
      }
    else
      if (p < 1)
	F20_printesc (366);
      else
	F17_print (p - 1);
  else
    if (p >= 3525)
      F20_printesc (366);
    else
      if ((hash[p - 258].rh < 0) || (hash[p - 258].rh >= strptr))
	F20_printesc (367);
      else
      {
	F20_printesc (207);
	F18_slowprint (hash[p - 258].rh);
	F16_printchar ((char)32);
      }
}

void F24_sprintcs (p)
/*************/
register int /* was T32_halfword */ p;
{
  if (p < 258)
    if (p < 129)
      F17_print (p - 1);
    else
      if (p < 257)
	F20_printesc (p - 129);
      else
      {
	F20_printesc (364);
	F20_printesc (365);
      }
  else
  {
    F20_printesc (207);
    F18_slowprint (hash[p - 258].rh);
  }
}

void F25_printfilename (n, a, e)
/**********************/
int   n;
int   a;
int   e;
{
  F17_print (a);
  F17_print (n);
  F17_print (e);
}

void F26_printsize (s)
/**************/
int   s;
{
  if (s == 0)
    F20_printesc (278);
  else
    if (s == 16)
      F20_printesc (279);
    else
      F20_printesc (280);
}

void F27_printwritewhatsit (s, p)
/************************/
T26_strnumber s;
register int /* was T32_halfword */ p;
{
  register int help;
  F20_printesc (s);
  help = mem[p + 1 - C2_memmin]._TAG._TAG3.hh._TAG._TAG1.lh;
  if (help < 16)
    F22_printint ((long)help);
  else
    if (help == 16)
      F16_printchar ((char)42);
    else
      F16_printchar ((char)45);
}

void F38_jumpout ()
/***********/
{
  longjmp (L_9998, 1);
}

void F39_error ()
/*********/
{
#ifdef VAX
register int c;
#else
  T21_asciicode c;
#endif
  register int   s1;
  register int   s2;
  register int   s3;
  register long int   s4;

  if (history < 2)
    history = 2;
  F16_printchar ((char)46);
  F31_showcontext ();
  if (interaction == 3)
    while (TRUE)
    {
  _22: 
      F35_clearforerrorprompt ();
      {
	F17_print (136);
	F30_terminput ();
      }
      if (last == first)
	return;
      c = buffer[first];
      if (c >= 97)
	c -= 32;
      switch (c)
      {
	case 48: 
	case 49: 
	case 50: 
	case 51: 
	case 52: 
	case 53: 
	case 54: 
	case 55: 
	case 56: 
	case 57: 
	  if (deletionsallowed)
	  {
	    s1 = curtok;
	    s2 = curcmd;
	    s3 = curchr;
	    s4 = alignstate;
	    alignstate = 1000000;
	    oktointerrupt = FALSE;
	    if (((last > (first + 1)) && (buffer[first + 1] >= 48)) && (buffer[first + 1] <= 57))
	      c = ((c * 10) + buffer[first + 1]) - (48 * 11);
	    else
	      c -= 48;
	    while (c > 0)
	    {
	      F29_gettoken (null);
	      c--;
	    }
	    curtok = s1;
	    curcmd = s2;
	    curchr = s3;
	    alignstate = s4;
	    oktointerrupt = TRUE;
	    {
	      helpptr = 2;
	      helpline[1] = 149;
	      helpline[0] = 150;
	    }
	    F31_showcontext ();
	    goto _22;
	  }
	   break;
# if defined(INITEX) || defined(TRIPTEX)
	case 68: 
	  F37_debughelp ();
	  goto _22;
	  break;
# endif
	case 69: 
	  if (baseptr > 0)
	  {
	    editnamestart = strstart[inputstack[baseptr].namefield];
	    V47_editnamelength = strstart[inputstack[baseptr].namefield + 1] - strstart[inputstack[baseptr].namefield];
	    editline = line;
	    F38_jumpout ();
	  }
	  break;
	case 72: 
	  if (useerrhelp)
	  {
	    F36_giveerrhelp ();
	    useerrhelp = FALSE;
	  }
	  else
	  {
	    if (helpptr == 0)
	    {
	      helpptr = 2;
	      helpline[1] = 151;
	      helpline[0] = 152;
	    }
	    do
	    {
	      F17_print (helpline[--helpptr]);
	      F15_println ();
	    }
	    while (helpptr != 0);
	  }
	  helpptr = 4;
	  helpline[3] = 153;
	  helpline[2] = 152;
	  helpline[1] = 154;
	  helpline[0] = 155;
	  goto _22;
	  break;
	case 73: 
	  F32_beginfilereading ();
	  if (last > (first + 1))
	  {
	    curinput.locfield = first + 1;
	    buffer[first] = 32;
	  }
	  else
	  {
	    F17_print (148);
	    F30_terminput ();
	    curinput.locfield = first;
	  }
	  first = last;
	  curinput.limitfield = last - 1;
	  return;
	  break;
	case 81: 
	case 82: 
	case 83: 
	  errorcount = 0;
	  interaction = c - 81;
	  F17_print (143);
	  switch (c)
	  {
	    case 81: 
	      F20_printesc (144);
	      selector--;
	      break;
	    case 82: 
	      F20_printesc (145);
	      break;
	    case 83: 
	      F20_printesc (146);
	      break;
	  }
	  F17_print (147);
	  F15_println ();
	  fflush(stdout);
	  return;
	  break;
	case 88: 
	  interaction = 2;
	  F38_jumpout ();
	  break;
	default: 
	  break;
      }
      F17_print (137);
      F19_printnl (138);
      F19_printnl (139);
      if (baseptr > 0)
	F17_print (140);
      if (deletionsallowed)
	F19_printnl (141);
      F19_printnl (142);
    }
  if (++errorcount == 100)
  {
    F19_printnl (135);
    history = 3;
    F38_jumpout ();
  }
  if (interaction > 0L)
    selector--;
  if (useerrhelp)
  {
    F15_println ();
    F36_giveerrhelp ();
  }
  else
    while (helpptr > 0)
    {
      helpptr--;
      F19_printnl (helpline[helpptr]);
    }
  F15_println ();
  if (interaction > 0)
    selector++;
  F15_println ();
}

void F40_fatalerror (s)
/***************/
T26_strnumber s;
{
  F28_normalizeselector ();
  F19_printnl (134);
  F17_print (157);
  helpptr = 1;
  helpline[0] = s;
  if (interaction == 3)
    interaction = 2;
  F39_error ();
#if defined(INITEX) || defined(TRIPTEX)
  if (interaction > 0)
    F37_debughelp ();
#endif
  history = 3;
  F38_jumpout ();
}

void F41_overflow (s, n)
/***************/
T26_strnumber s;
long int   n;
{
  F28_normalizeselector ();
  F19_printnl (134);
  F17_print (158);
  F17_print (s);
  F16_printchar ((char)61);
  F22_printint (n);
  F16_printchar ((char)93);
  helpptr = 2;
  helpline[1] = 159;
  helpline[0] = 160;
  if (interaction == 3)
    interaction = 2;
  F39_error ();
# if defined(INITEX) || defined(TRIPTEX)
  if (interaction > 0)
    F37_debughelp ();
# endif
  history = 3;
  F38_jumpout ();
}

void F42_confusion (s)
/**************/
T26_strnumber s;
{
  F28_normalizeselector ();
  if (history < 2)
  {
    F19_printnl (134);
    F17_print (161);
    F17_print (s);
    F16_printchar ((char)41);
    helpptr = 1;
    helpline[0] = 162;
  }
  else
  {
    F19_printnl (134);
    F17_print (163);
    helpptr = 2;
    helpline[1] = 164;
    helpline[0] = 165;
  }
  if (interaction == 3)
    interaction = 2;
  F39_error ();
# if defined(INITEX) || defined(TRIPTEX)
  if (interaction > 0)
    F37_debughelp ();
# endif
  history = 3;
  F38_jumpout ();
}
