
/*
 *    Copyright 1986, 1987 Pat Joseph Monardo. All rights reserved.
 *    Copying of this file is granted according to the provisions 
 *    specified in the file COPYING which must accompany this file.
 */


/*
 *		pack.h
 */

#define	EXACTLY				0
#define	ADDITIONAL			1
#define	NATURAL				0L, ADDITIONAL

global	ptr		adjust_tail;
global	scal	total_stretch[];
global	scal	total_shrink[];

global	long	pack_begin_line;

#define	make_char_from_lig() \
	{mem[lig_trick] = mem[lig_char(p)]; \
	link(lig_trick) = link(p); \
	p = lig_trick;}

#define	get_stretch_order() \
	{if (total_stretch[FILLL] != 0) o = FILLL; \
	else if (total_stretch[FILL] != 0) o = FILL; \
	else if (total_stretch[FIL] != 0) o = FIL; \
	else o = NORMAL;}
			
#define	get_shrink_order() \
	{if (total_shrink[FILLL] != 0) o = FILLL; \
	else if (total_shrink[FILL] != 0) o = FILL; \
	else if (total_shrink[FIL] != 0) o = FIL; \
	else o = NORMAL;}

#define	vpack(P, H) 		vpackage(P, H, MAX_DIMEN)
ptr		vpackage();
ptr		hpack();
