
/*
 *    Copyright 1986, 1987 Pat Joseph Monardo. All rights reserved.
 *    Copying of this file is granted according to the provisions 
 *    specified in the file COPYING which must accompany this file.
 */


/*
 *		arith.h
 */

#define	UNITY			0200000
#define	TWO				0400000

val		half();
scal	round_decimals();
int		print_scaled();

global	bool	arith_error;
global	scal	remainder;

scal	nx_plus_y();
scal	x_over_n();
scal	xn_over_d();

#define	INF_BAD			10000 

hword	badness();
