cl -c -M2l -DDOS ..\CHAR.C >> errs
cl -c -M2l -DDOS ..\IO.C >> errs
cl -c -M2l -DDOS ..\STR.C >> errs
cl -c -M2l -DDOS ..\PRINT.C >> errs
cl -c -M2l -DDOS ..\ERROR.C >> errs
cl -c -M2l -DDOS ..\ARITH.C >> errs
cl -c -M2l -DDOS ..\HEAP.C >> errs
cl -c -M2l -DDOS ..\BOX.C >> errs
cl -c -M2l -Od -DDOS ..\CMDS.C > errs
cl -c -M2l -DDOS ..\EVALSTAC.C >> errs
cl -c -M2l -DDOS ..\EQ.C >> errs
cl -c -M2l -DDOS ..\HASH.C >> errs
cl -c -M2l -DDOS ..\EQSTACK.C >> errs
cl -c -M2l -DDOS ..\TOKENSTA.C >> errs
cl -c -M2l -Od -DDOS ..\TOKEN.C >> errs
cl -c -M2l -DDOS ..\EXPAND.C >> errs
cl -c -M2l -DDOS ..\SCAN.C >> errs
cl -c -M2l -DDOS ..\TOKENLIS.C >> errs
cl -c -M2l -DDOS ..\COND.C >> errs
cl -c -M2l -DDOS ..\FILE.C >> errs
cl -c -M2l -DDOS ..\TFM.C >> errs
cl -c -M2l -DDOS ..\DVI.C >> errs
cl -c -M2l -DDOS ..\PACK.C >> errs
cl -c -M2l -DDOS ..\MATH.C >> errs
cl -c -M2l -DDOS ..\MLIST-HL.C >> errs
cl -c -M2l -DDOS ..\ALIGN.C >> errs
cl -c -M2l -DDOS ..\PAR.C >> errs
cl -c -M2l -DDOS ..\HYPH.C >> errs
cl -c -M2l -DDOS ..\PAGE.C >> errs
cl -c -M2l -DDOS ..\EVAL.C >> errs
cl -c -M2l -DDOS ..\BOXLISTS.C >> errs
cl -c -M2l -DDOS ..\MATHLIST.C >> errs
cl -c -M2l -DDOS ..\DEF.C >> errs
cl -c -M2l -DDOS ..\FMT.C >> errs
cl -c -M2l -DDOS ..\TEX.C >> errs
cl -c -M2l -DDOS ..\TEXEXT.C >> errs
link @virtex.lnk >> errs
